@@ -1,13 +1,7 @@
--- project
 set_project("tbox")
-
--- set xmake minimum version
-set_xmakever("2.8.2")
-
--- set project version
-set_version("1.7.6", {build = "%Y%m%d", soname = true})
-
--- set warning all as error
+set_xmakever("3.0.5")
+set_policy("build.progress_style", "multirow")
+set_version("1.7.9", {build = "%Y%m%d", soname = true})
 set_warnings("all", "error")
 
 -- set language: c99
@@ -18,6 +12,12 @@ set_languages(stdc)
 set_configvar("_GNU_SOURCE", 1)
 set_configvar("_REENTRANT", 1)
 
+-- ensure POSIX/XOPEN features are available on Solaris (for setenv, unsetenv, clock_gettime, etc.)
+-- _XOPEN_SOURCE=600 implicitly sets _POSIX_C_SOURCE to 200112L
+if is_plat("solaris") then
+    add_defines("_XOPEN_SOURCE=600")
+end
+
 -- add module directories
 add_moduledirs("xmake")
 
